@@ -1028,17 +1028,21 @@ nautilus_burn_iso_graft_get_info (NautilusBurnIsoGraft *graft,
 		g_error_free (sub_error);
 
 		g_ptr_array_free (argv, TRUE);
+		argv = NULL;
 
 		result = FALSE;
 		goto done;
 	}
 
 	g_ptr_array_free (argv, TRUE);
+	argv = NULL;
 
 	if (exit_status != 0 && use_joliet) {
 		if (strstr (stderr_data, "Joliet tree sort failed.") != NULL) {
 			g_free (stdout_data);
 			g_free (stderr_data);
+			stdout_data = NULL;
+			stderr_data = NULL;
 			if (ask_disable_joliet (NULL)) {
 				use_joliet = FALSE;
 				goto retry;
@@ -1237,31 +1241,28 @@ nautilus_burn_iso_make_from_graft (NautilusBurnIso             *iso,
 
 	if (res == -1) {
 		g_warning ("Cannot get free space at %s\n", dirname);
-	} else if (iso_size > size) {
-		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
-			char *message;
-
-			message = get_image_space_error_message (iso_size,
-								 size);
-
-			g_set_error (error,
-				     NAUTILUS_BURN_ISO_ERROR,
-				     NAUTILUS_BURN_ISO_ERROR_GENERAL,
-				     "%s", message);
-			g_free (message);
-
-			result = NAUTILUS_BURN_ISO_RESULT_ERROR;
-		} else {
-			result = NAUTILUS_BURN_ISO_RESULT_RETRY;
-		}
-
-		g_free (dirname);
-
-		goto cleanup;
+		size = 0;
 	}
 
 	g_free (dirname);
 
+	if (iso_size > size) {
+		char *message;
+
+		message = get_image_space_error_message (iso_size,
+							 size);
+
+		g_set_error (error,
+			     NAUTILUS_BURN_ISO_ERROR,
+			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+			     "%s", message);
+		g_free (message);
+
+		result = NAUTILUS_BURN_ISO_RESULT_RETRY;
+
+		goto cleanup;
+	}
+
 	argv = g_ptr_array_new ();
 	g_ptr_array_add (argv, "mkisofs");
 	g_ptr_array_add (argv, "-r");
@@ -1517,26 +1518,26 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 
 	if (res == -1) {
 		g_warning ("Cannot get free space at %s\n", dirname);
-		g_free (dirname);
-	} else if (iso_size > size) {
-		g_free (dirname);
-		if (flags & NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE) {
-			char *message;
 
-			message = get_image_space_error_message (iso_size,
-								 size);
+		size = 0;
+	}
 
-			g_set_error (error, NAUTILUS_BURN_ISO_ERROR,
-				     NAUTILUS_BURN_ISO_ERROR_GENERAL,
-				     "%s", message);
-			g_free (message);
+	g_free (dirname);
 
-			result = NAUTILUS_BURN_ISO_RESULT_ERROR;
-		} else {
-			result = NAUTILUS_BURN_ISO_RESULT_RETRY;
-		}
+	if (iso_size > size) {
+		char *message;
 
-		return result;
+		message = get_image_space_error_message (iso_size,
+							 size);
+
+		g_set_error (error, NAUTILUS_BURN_ISO_ERROR,
+			     NAUTILUS_BURN_ISO_ERROR_GENERAL,
+			     "%s", message);
+		g_free (message);
+
+		result = NAUTILUS_BURN_ISO_RESULT_RETRY;
+
+		goto cleanup;
 	}
 
 	filename_arg = NULL;
@@ -1587,6 +1588,8 @@ nautilus_burn_iso_make_from_drive (NautilusBurnIso             *iso,
 	g_free (filename_arg);
 	g_free (toc_filename_arg);
 
+ cleanup:
+
 	return result;
 }
 
