@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
@@ -33,6 +34,7 @@
 #include <gconf/gconf-client.h>
 
 #include "nautilus-burn-drive.h"
+#include "nautilus-burn-drive-monitor.h"
 #include "nautilus-burn-recorder.h"
 #include "make-iso.h"
 
@@ -138,9 +140,9 @@ ncb_hig_dialog (GtkMessageType type,
 					type,
 					GTK_BUTTONS_NONE,
 					title);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog), reason);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog), "%s", reason);
 	gtk_window_set_title (GTK_WINDOW (error_dialog), "");
-	gtk_window_set_icon_name (GTK_WINDOW (error_dialog), "gnome-dev-cdrom");
+	gtk_window_set_icon_name (GTK_WINDOW (error_dialog), "nautilus-cd-burner");
 
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 
@@ -166,7 +168,26 @@ static void
 ncb_operation_temp_files_add (NcbOperation *operation,
 			      const char   *filename)
 {
-	operation->priv->files_for_cleanup = g_list_append (operation->priv->files_for_cleanup, g_strdup (filename));
+	if (filename != NULL) {
+		operation->priv->files_for_cleanup = g_list_append (operation->priv->files_for_cleanup, g_strdup (filename));
+	}
+}
+
+static void
+remove_file (const char *filename)
+{
+	int res;
+
+	if (filename == NULL) {
+		return;
+	}
+
+	res = g_remove (filename);
+	if (res != 0) {
+		g_warning ("Could not remove file %s: %s",
+			   filename,
+			   g_strerror (errno));
+	}
 }
 
 static void
@@ -177,10 +198,7 @@ ncb_operation_temp_files_cleanup (NcbOperation *operation)
 	while (operation->priv->files_for_cleanup != NULL) {
 		char *filename = operation->priv->files_for_cleanup->data;
 
-		if (filename) {
-			g_unlink (filename);
-		}
-
+		remove_file (filename);
 		g_free (filename);
 
 		operation->priv->files_for_cleanup = g_list_remove (operation->priv->files_for_cleanup,
@@ -278,7 +296,7 @@ size_to_string_text (guint64 bytes)
 	guint64     t_bytes;
 	const char *unit;
 	char       *result;
-	
+
 	if (bytes > 1125899906842624ull) {
 		t_bytes = (bytes * 10) / 1125899906842624ull;
 		unit = "PiB";
@@ -298,13 +316,13 @@ size_to_string_text (guint64 bytes)
 		t_bytes = bytes;
 		unit = "B";
 	}
-	
+
 	result = g_strdup_printf ("%" G_GUINT64_FORMAT ".%" G_GUINT64_FORMAT " %s", t_bytes / 10, t_bytes % 10, unit);
 
 	return result;
 }
 
-static int  
+static int
 ask_for_media (NcbOperation      *operation,
 	       NautilusBurnDrive *drive,
 	       gboolean           is_reload,
@@ -323,7 +341,10 @@ ask_for_media (NcbOperation      *operation,
 
 	size_string = size_to_string_text ((guint64)required_size);
 
-	type_string = nautilus_burn_drive_get_supported_media_string (drive, TRUE);
+	type_string = nautilus_burn_drive_get_supported_media_string_for_size (drive, (guint64)required_size);
+	if (type_string == NULL) {
+		type_string = g_strdup (_("None"));
+	}
 
 	can_rewrite = nautilus_burn_drive_can_rewrite (drive);
 	show_eject = FALSE;
@@ -411,7 +432,7 @@ insert_media_request_cb (NautilusBurnRecorder *recorder,
 	if (res == GTK_RESPONSE_CANCEL) {
 		return FALSE;
 	}
- 
+
 	return TRUE;
 }
 
@@ -435,7 +456,7 @@ ask_rewrite_disc (NcbOperation *operation)
 
 	dialog = ncb_hig_dialog (GTK_MESSAGE_WARNING,
 				 _("Erase information on this disc?"),
-				 msg, 
+				 msg,
 				 GTK_WINDOW (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog)));
 	g_free (msg);
 
@@ -509,7 +530,7 @@ reset_progress_dialog (NcbOperation *operation)
 
 static void
 do_eject (NcbOperation *operation)
-	       
+
 {
 	NautilusBurnDrive *drive;
 
@@ -550,13 +571,13 @@ finish_progress_dialog (NcbOperation *operation)
 	ncb_progress_dialog_done (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog));
 
 	/* If target is not a disc then don't offer eject */
-	if (drive->device == NULL) {
+	if (nautilus_burn_drive_get_device (drive) == NULL) {
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (operation->priv->progress_dialog),
 						   GTK_RESPONSE_REJECT, FALSE);
 	}
 }
 
-static gboolean 
+static gboolean
 overwrite_existing_file (GtkWindow  *filesel,
 			 const char *filename)
 {
@@ -606,7 +627,7 @@ select_iso_filename (const char *default_name)
  reselect:
 	filename = NULL;
 	response = gtk_dialog_run (GTK_DIALOG (chooser));
-	
+
 	if (response == GTK_RESPONSE_OK) {
 		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 		if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
@@ -625,10 +646,10 @@ select_iso_filename (const char *default_name)
 static gboolean
 request_media (NcbOperation      *operation,
 	       NautilusBurnDrive *drive,
-	       gint64             required_size,
+	       gint64             required_capacity,
 	       gboolean           request_overburn)
 {
-	gint64                media_size;
+	gint64                media_capacity;
 	gboolean              reload;
 	NautilusBurnMediaType type;
 	gboolean              is_rewritable;
@@ -638,7 +659,7 @@ request_media (NcbOperation      *operation,
 	gboolean              has_audio;
 
 	/* If the output is a file no need to prompt */
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		return TRUE;
 	}
 
@@ -651,23 +672,23 @@ request_media (NcbOperation      *operation,
 		gboolean res;
 		res = nautilus_burn_drive_unmount (drive);
 		if (! res) {
-			g_warning ("Couldn't unmount volume in drive: %s", drive->device);
+			g_warning ("Couldn't unmount volume in drive: %s", nautilus_burn_drive_get_device (drive));
 		}
 	}
 
-	/* if overburn requested then no need to check size/type */
+	/* if overburn requested then no need to check capacity/type */
 	if (request_overburn) {
 		return TRUE;
 	}
 
-	media_size = nautilus_burn_drive_get_media_size (drive);
+	media_capacity = nautilus_burn_drive_get_media_capacity (drive);
 	type = nautilus_burn_drive_get_media_type_full (drive, &is_rewritable, &is_blank, &has_data, &has_audio);
 	can_write = nautilus_burn_drive_media_type_is_writable (type, is_blank);
 
-	if ((! can_write) || (required_size > media_size)) {
+	if ((! can_write) || (required_capacity > media_capacity)) {
 		int res;
 
-		reload = (media_size > 0);
+		reload = (media_capacity > 0);
 		if (type == NAUTILUS_BURN_MEDIA_TYPE_ERROR) {
 			reload = FALSE;
 		}
@@ -676,7 +697,7 @@ request_media (NcbOperation      *operation,
 				     drive,
 				     reload,
 				     FALSE,
-				     required_size);
+				     required_capacity);
 
 		if (res == GTK_RESPONSE_CANCEL) {
 			return FALSE;
@@ -699,7 +720,7 @@ sanitize_filename (const char *path,
 	while (*path == '.') {
 		path++;
 	}
-  
+
 	ret = g_strdup (path);
 	/* Replace path seperators with a hyphen */
 	g_strdelimit (ret, "/", '-');
@@ -952,7 +973,7 @@ update_progress_dialog_disc_type (NcbProgressDialog        *dialog,
 
 	input_type_index = get_input_type_index (source_type);
 
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		output_type = NCB_OUTPUT_FILE;
 	} else {
 		switch (type) {
@@ -997,7 +1018,7 @@ maybe_request_media (NcbOperation             *operation,
 {
 	NautilusBurnMediaType _media_type = NAUTILUS_BURN_MEDIA_TYPE_UNKNOWN;
 
-	if (drive->type != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		/* if we are reading from a drive then don't request media yet */
 		if (source_type != NCB_SELECTION_SOURCE_DEVICE) {
 			gboolean              is_locked;
@@ -1012,7 +1033,7 @@ maybe_request_media (NcbOperation             *operation,
 
 			if (is_locked) {
 				nautilus_burn_drive_unlock (drive);
-			} 
+			}
 
 			if (! got_media) {
 				return FALSE;
@@ -1058,9 +1079,8 @@ build_temp_iso_filename (const char *path)
 }
 
 static char *
-get_temp_filename_gconf (void)
+get_temp_path_gconf (void)
 {
-	char        *filename;
 	char        *path;
 	GConfClient *gc;
 
@@ -1068,25 +1088,28 @@ get_temp_filename_gconf (void)
 	path = gconf_client_get_string (gc, "/apps/nautilus-cd-burner/temp_iso_dir", NULL);
 	g_object_unref (gc);
 
-	filename = NULL;
+	return path;
+}
+
+static char **
+get_temp_paths (void)
+{
+	char      *path;
+	GPtrArray *arr;
+
+	arr = g_ptr_array_new ();
+
+	path = get_temp_path_gconf ();
 	if (path != NULL && path[0] != '\0') {
-		filename = build_temp_iso_filename (path);
+		g_ptr_array_add (arr, g_strdup (path));
 	}
 	g_free (path);
 
-	return filename;
-}
+	g_ptr_array_add (arr, g_strdup (g_get_tmp_dir ()));
+	g_ptr_array_add (arr, g_strdup (g_get_home_dir ()));
+	g_ptr_array_add (arr, NULL);
 
-static char *
-get_temp_filename_tmp (void)
-{
-	return build_temp_iso_filename (g_get_tmp_dir ());
-}
-
-static char *
-get_temp_filename_home (void)
-{
-	return build_temp_iso_filename (g_get_home_dir ());
+	return (char **)g_ptr_array_free (arr, FALSE);
 }
 
 static NautilusBurnRecorderTrack *
@@ -1115,20 +1138,14 @@ create_track_from_device (NcbOperation *operation,
 	NautilusBurnDrive           *source_drive;
 	NautilusBurnDrive           *drive;
 	char                        *label;
-	int                          i;
-	int                          j;
-	char                        *temp_files[4];
 	char                        *iso_filename;
 	char                        *toc_filename;
+	char                       **temp_paths;
+	int                          i;
 
 	track = NULL;
 	filename = NULL;
-
-	i = 0;
-	temp_files[i++] = get_temp_filename_gconf ();
-	temp_files[i++] = get_temp_filename_tmp ();
-	temp_files[i++] = get_temp_filename_home ();
-	temp_files[i++] = NULL;
+	temp_paths = NULL;
 
 	operation->priv->cancel = CANCEL_MAKE_ISO;
 
@@ -1140,11 +1157,12 @@ create_track_from_device (NcbOperation *operation,
 	}
 
 	ncb_selection_get_label (operation->priv->selection, &label);
-	source_drive = nautilus_burn_drive_new_from_path (source_name);
+	source_drive = nautilus_burn_drive_monitor_get_drive_for_device (nautilus_burn_get_drive_monitor (),
+									 source_name);
 
 	iso_filename = NULL;
 	drive = ncb_selection_peek_drive (operation->priv->selection);
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		iso_filename = get_output_filename (label);
 
 		if (iso_filename == NULL) {
@@ -1164,23 +1182,21 @@ create_track_from_device (NcbOperation *operation,
 	ncb_progress_dialog_set_operation_string (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog), _("Creating disc image"));
 
 	toc_filename = NULL;
-	filename = NULL;
+	filename     = NULL;
+	res          = NAUTILUS_BURN_ISO_RESULT_RETRY;
+	temp_paths   = get_temp_paths ();
+	for (i = 0; temp_paths [i] != NULL; i++) {
 
-	if (iso_filename != NULL) {
-		/* Don't warn on low space unless the user selected this file specifically */
-		image_flags |= NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE;
-	}
-
-	res = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
-	for (j = 0; j < i; j++) {
 		if (iso_filename == NULL) {
-			filename = g_strdup (temp_files[j]);
+			filename = build_temp_iso_filename (temp_paths [i]);
 		} else {
 			filename = g_strdup (iso_filename);
 		}
+
 		if (filename == NULL) {
 			continue;
 		}
+
 		res = nautilus_burn_iso_make_from_drive (operation->priv->iso,
 							 filename,
 							 source_drive,
@@ -1188,9 +1204,20 @@ create_track_from_device (NcbOperation *operation,
 							 &image_type,
 							 &toc_filename,
 							 error);
-		if (res != NAUTILUS_BURN_RECORDER_RESULT_RETRY) {
+		if (res != NAUTILUS_BURN_ISO_RESULT_RETRY) {
 			break;
 		}
+
+		/* if this was the last try then save the error */
+		if (temp_paths [i + 1] != NULL) {
+			g_clear_error (error);
+		}
+
+		/* remove temporary file */
+		if (iso_filename == NULL) {
+			remove_file (filename);
+		}
+
 		g_free (filename);
 		filename = NULL;
 		g_free (toc_filename);
@@ -1201,31 +1228,23 @@ create_track_from_device (NcbOperation *operation,
 		goto done;
 	}
 
+	/* track takes ownership of file */
 	track = create_iso_track (filename);
 
  done:
-	for (j = 0; j < i; j++) {
-		if (temp_files[j] == NULL) {
-			continue;
-		}
 
-		if (filename != NULL && strcmp (temp_files[j], filename) != 0) {
-			g_remove (temp_files[j]);
-		}
-		g_free (temp_files[j]);
-	}
 	if (operation->priv->iso != NULL) {
 		g_object_unref (operation->priv->iso);
 		operation->priv->iso = NULL;
 	}
 
+	g_strfreev (temp_paths);
 	g_free (toc_filename);
 	g_free (filename);
 	g_free (iso_filename);
 
 	g_free (label);
 
-	nautilus_burn_drive_eject (source_drive);
 	g_object_unref (source_drive);
 
 	operation->priv->cancel = CANCEL_NONE;
@@ -1261,22 +1280,16 @@ create_track_from_burn_folder (NcbOperation *operation,
 	char                        *filename;
 	NautilusBurnDrive           *drive;
 	char                        *label;
-	int                          i;
-	int                          j;
-	char                        *temp_files[4];
 	char                        *iso_filename;
 	NautilusBurnIsoGraft        *graft;
 	NautilusBurnMediaType        media_type;
 	guint64                      source_size;
+	char                       **temp_paths;
+	int                          i;
 
 	track = NULL;
 	filename = NULL;
-
-	i = 0;
-	temp_files[i++] = get_temp_filename_gconf ();
-	temp_files[i++] = get_temp_filename_tmp ();
-	temp_files[i++] = get_temp_filename_home ();
-	temp_files[i++] = NULL;
+	temp_paths = NULL;
 
 	operation->priv->cancel = CANCEL_MAKE_ISO;
 
@@ -1291,7 +1304,7 @@ create_track_from_burn_folder (NcbOperation *operation,
 
 	iso_filename = NULL;
 	drive = ncb_selection_peek_drive (operation->priv->selection);
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		iso_filename = get_output_filename (label);
 
 		if (iso_filename == NULL) {
@@ -1315,10 +1328,9 @@ create_track_from_burn_folder (NcbOperation *operation,
 			  operation);
 	ncb_progress_dialog_set_operation_string (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog), _("Creating disc image"));
 
-
 	graft = nautilus_burn_iso_graft_new (BURN_URI);
 
-	if (drive->type != NAUTILUS_BURN_DRIVE_TYPE_FILE
+	if (nautilus_burn_drive_get_drive_type (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE
 	    && NAUTILUS_BURN_DRIVE_MEDIA_TYPE_IS_DVD (media_type)) {
 		GList   *l;
 		gboolean use_joliet;
@@ -1342,22 +1354,22 @@ create_track_from_burn_folder (NcbOperation *operation,
 		/* Don't remove the files */
 		nautilus_burn_iso_graft_free (graft, FALSE);
 	} else {
-		if (iso_filename != NULL) {
-			/* Don't warn on low space unless the user selected this file specifically */
-			image_flags |= NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE;
-		}
 
 		res = NAUTILUS_BURN_ISO_RESULT_ERROR;
 
-		for (j = 0; j < i; j++) {
+		temp_paths = get_temp_paths ();
+		for (i = 0; temp_paths [i] != NULL; i++) {
+
 			if (iso_filename == NULL) {
-				filename = g_strdup (temp_files[j]);
+				filename = build_temp_iso_filename (temp_paths [i]);
 			} else {
 				filename = g_strdup (iso_filename);
 			}
+
 			if (filename == NULL) {
 				continue;
 			}
+
 			res = nautilus_burn_iso_make_from_graft (operation->priv->iso,
 								 graft,
 								 filename,
@@ -1365,9 +1377,21 @@ create_track_from_burn_folder (NcbOperation *operation,
 								 image_flags,
 								 &image_type,
 								 error);
+
 			if (res != NAUTILUS_BURN_ISO_RESULT_RETRY) {
 				break;
 			}
+
+			/* if this was the last try then save the error */
+			if (temp_paths [i + 1] != NULL) {
+				g_clear_error (error);
+			}
+
+			/* remove temporary file */
+			if (iso_filename == NULL) {
+				remove_file (filename);
+			}
+
 			g_free (filename);
 			filename = NULL;
 		}
@@ -1378,7 +1402,7 @@ create_track_from_burn_folder (NcbOperation *operation,
 		}
 
 		/* if we aren't writing to an ISO schedule this temp file for removal */
-		if (drive->type != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+		if (nautilus_burn_drive_get_drive_type (drive) != NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 			ncb_operation_temp_files_add (operation, filename);
 		}
 
@@ -1387,16 +1411,7 @@ create_track_from_burn_folder (NcbOperation *operation,
 	}
 
  done:
-	/* remove temp files */
-	for (j = 0; j < i; j++) {
-		if (temp_files[j] == NULL) {
-			continue;
-		}
-		if (filename != NULL && strcmp (temp_files[j], filename) != 0) {
-			g_remove (temp_files[j]);
-		}
-		g_free (temp_files[j]);
-	}
+	g_strfreev (temp_paths);
 
 	if (operation->priv->iso != NULL) {
 		g_object_unref (operation->priv->iso);
@@ -1515,18 +1530,24 @@ burn_cd (NcbOperation *operation)
 {
 	int                res;
 	NautilusBurnDrive *drive;
+	NcbSelectionSource source_type;
+	char              *source_name;
 
 	drive = ncb_selection_peek_drive (operation->priv->selection);
 
 	if (! prepare_tracks (operation)) {
 		res = NAUTILUS_BURN_RECORDER_RESULT_CANCEL;
+		g_warning ("Unable to prepare tracks for burning");
 		goto out;
 	}
 
-	if (drive->type == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
+	if (nautilus_burn_drive_get_drive_type (drive) == NAUTILUS_BURN_DRIVE_TYPE_FILE) {
 		NautilusBurnRecorderTrack *track;
 		char                      *str;
 
+		/* don't show make another button */
+		ncb_progress_dialog_set_make_another_button_visible (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog), FALSE);
+
 		finish_progress_dialog (operation);
 
 		track = operation->priv->tracks->data;
@@ -1540,6 +1561,21 @@ burn_cd (NcbOperation *operation)
 		goto out;
 	}
 
+	/* special case: when copying a disc eject the source if same as destination */
+	source_name = NULL;
+	ncb_selection_get_source (operation->priv->selection, &source_type, &source_name);
+	if (source_type == NCB_SELECTION_SOURCE_DEVICE) {
+		NautilusBurnDrive *source_drive;
+
+		source_drive = nautilus_burn_drive_monitor_get_drive_for_device (nautilus_burn_get_drive_monitor (),
+										 source_name);
+		if (source_drive != NULL
+		    && nautilus_burn_drive_equal (drive, source_drive)) {
+			nautilus_burn_drive_eject (source_drive);
+			g_object_unref (source_drive);
+		}
+	}
+
 	res = write_disc (operation);
 
  out:
@@ -1700,7 +1736,8 @@ progress_dialog_response (GtkWidget    *dialog,
 		do_callback (operation);
 	}
 
-	if (response_id == GTK_RESPONSE_CANCEL) {
+	if (response_id == GTK_RESPONSE_CANCEL
+	    || response_id == GTK_RESPONSE_DELETE_EVENT) {
 		if (! handle_cancel (operation)) {
 			gtk_widget_hide (dialog);
 			do_callback (operation);
